<?php

	namespace Drupal\idealcheckout\Plugin\Commerce\PaymentGateway;

	use Drupal\commerce_payment\Exception\PaymentGatewayException;
	use Drupal\commerce_payment\Exception\DeclineException;
	use Drupal\commerce_payment\Exception\AuthenticationException;
	use Drupal\commerce_payment\Exception\InvalidResponseException;
	use Drupal\Core\Entity\EntityTypeManagerInterface;
	use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
	use Drupal\commerce_order\Entity\OrderInterface;
	use Drupal\Core\Form\FormStateInterface;
	use Symfony\Component\HttpFoundation\Request;


	/**
	 *
	 * @CommercePaymentGateway(
	 *   id = "iDEAL Checkout",
	 *   label = @Translation("iDEAL Checkout"),
	 *   display_label = @Translation("iDEAL Checkout"),
	 *    forms = {
	 *     "offsite-payment" = "Drupal\idealcheckout\PluginForm\idealcheckout_redirect",
	 *   },
	 * )
	 */

	class idealcheckout extends OffsitePaymentGatewayBase
	{
/**
		* {@inheritdoc}
		*/
		public function defaultConfiguration()
		{
			return [
			  'configuration' => [
				'status' => 0,
				'payment_methods' => 'ideal',
			  ],
			];
		}

		/**
		* {@inheritdoc}
		*/
		public function buildConfigurationForm(array $form, FormStateInterface $form_state)
		{
			$form = parent::buildConfigurationForm($form, $form_state);

			$form['configuration'] = [
			  '#type' => 'details',
			  '#title' => $this->t('Configuration settings'),
			  '#open' => TRUE,
			];
			/*
			$form['configuration']['status'] = [
			  '#type' => 'select',
			  '#required' => TRUE,
			  '#title' => $this->t('Default status for unfinished order'),
			  '#options' => [
				0 => $this->t('Pending'),
			  ],
			];
			*/
			$form['configuration']['payment_methods'] = [
			  '#type' => 'select',
			  '#required' => TRUE,
			  '#title' => $this->t('Payment Method'),
			  '#options' => [
				'afterpay' => $this->t('AFTERPAY'),
				'creditcard' => $this->t('CREDITCARD'),
				'directebanking' => $this->t('DIRECTEBANKING'),
				'sofortbanking' => $this->t('SOFORTBANKING'),
				'ideal' => $this->t('IDEAL'),
				'klarnaaccount' => $this->t('KLARNAACCOUNT'),
				'klarnainvoice' => $this->t('KLARNAINVOICE'),
				'klarnabuynow' => $this->t('KLARNABUYNOW'),
				'maestro' => $this->t('MAESTRO'),
				'manual_transfer' => $this->t('MANUAL TRANSFER'),
				'mastercard' => $this->t('MASTERCARD'),
				'mistercash' => $this->t('MISTERCASH'),
				'bancontact' => $this->t('BANCONTACT'),
				'payconiq' => $this->t('PAYCONIQ'),
				'paypal' => $this->t('PAYPAL'),
				'paysafecard' => $this->t('PAYSAFECARD'),
				'visa' => $this->t('VISA'),
				'vpay' => $this->t('VPAY'),
			  ],
			];

			return $form;
		}

		/**
		* {@inheritdoc}
		*/
		public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
		{
			parent::validateConfigurationForm($form, $form_state);
		}

		/**
		* {@inheritdoc}
		*/
		public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
		{
			parent::submitConfigurationForm($form, $form_state);
			if (!$form_state->getErrors()) {
			  $values = $form_state->getValue($form['#parents']);
			  $this->configuration['configuration'] = $values['configuration'];
			}
		}

		/**
		* {@inheritdoc}
		*/
		public function onReturn(OrderInterface $order, Request $request)
		{

			$sOrderId = $order->id();
			$sTransactionStatus = \Drupal\idealcheckout\idealcheckout::getStatus($sOrderId);

			if ($sTransactionStatus == 'SUCCESS')
			{
				$payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
				$payment = $payment_storage->create([
				  'state' => 'Transaction completed',
				  'amount' => $order->getBalance(),
				  'payment_gateway' => $this->entityId,
				  'order_id' => $order->id(),
				  'remote_id' => $request->request->get('txn_id'),
				  'remote_state' => $request->request->get('payment_status'),
				]);
				$payment->save();
			}
			elseif($sTransactionStatus == 'PENDING')
			{
				throw new PaymentGatewayException('Payment failed!');
				
				$payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
				$payment = $payment_storage->create([
				  'state' => 'Transaction pending',
				  'amount' => $order->getBalance(),
				  'payment_gateway' => $this->entityId,
				  'order_id' => $order->id(),
				  'remote_id' => $request->request->get('txn_id'),
				  'remote_state' => $request->request->get('payment_status'),
				]);
				$payment->save();
			}
			elseif($sTransactionStatus == 'FAILURE')
			{
				throw new PaymentGatewayException('Payment failed!');
				
				$payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
				$payment = $payment_storage->create([
				  'state' => 'Transaction failed',
				  'amount' => $order->getBalance(),
				  'payment_gateway' => $this->entityId,
				  'order_id' => $order->id(),
				  'remote_id' => $request->request->get('txn_id'),
				  'remote_state' => $request->request->get('payment_status'),
				]);
				$payment->save();
			}
		}
	}
?>