<?php
	namespace Drupal\idealcheckout\PluginForm;

	use Drupal\Core\Form\FormStateInterface;
	use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
	use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
	use Drupal\idealcheckout\idealcheckout;

	class idealcheckout_redirect extends BasePaymentOffsiteForm
	{

	  /**
	   * {@inheritdoc}
	   */
		public function buildConfigurationForm(array $form, FormStateInterface $form_state)
		{
			$form = parent::buildConfigurationForm($form, $form_state);

			/** @var \Drupal\commerce_payment\Entity\PaymentInterface $oPayment */
			$oPayment = $this->entity; // the payment opbject
			
			$sOrderId = $oPayment->getOrderId();
			$oEntityManager = \Drupal::entityTypeManager(); // the entityTypeManager use this for entitys.
			$oOrder = $oEntityManager->getStorage('commerce_order')->load($sOrderId); // The order based on the order id
			
			$sPaymentMethod = $oPayment->getPaymentGatewayId(); // Used Payment method

			$fAmount = $oOrder->getTotalPrice()->getNumber(); // Cart total

			$sUrl = $form['#return_url']; // return url. Activates the onReturn function in Plugin\Commerce\PaymentGateway\idealcheckout.php

			$sReturnUrl = $form['#return_url'];	// set the Return url
			$sFailureUrl = $form['#cancel_url']; // set the Cancel / failure URL

			$oIdealcheckout = new idealcheckout($fAmount, $sPaymentMethod, $sOrderId); // creates the global idealcheckout object

			$sOrderCode = $oIdealcheckout->getOrderCode(); // returns the ordercode generated in $oIdealcheckout

			$oIdealcheckout->setUrls($sUrl);

			$oIdealcheckout->setTransactionFailureUrl($sFailureUrl . '?order_id=' . $sOrderId . '&order_code=' . $sOrderCode);


			$sEmailAddress = $oOrder->getEmail(); // Returns the emailadress from the customer.

			// Get Billing Data
			$aBillingInformation = $oOrder->billing_profile->entity->address->getValue(); // Get the billing address and billing information

			// Set Billing Data

			$sBillingCountryCode = $aBillingInformation[0]['country_code'];
			$sBillingCity = $aBillingInformation[0]['locality'];
			$sBillingZip = $aBillingInformation[0]['postal_code'];
			$sBillingStreetName = $aBillingInformation[0]['address_line1'];
			$sBillingHouseNumber = $aBillingInformation[0]['address_line2'];
			$sBillingCompany = $aBillingInformation[0]['organization'];
			$sBillingFirstName = $aBillingInformation[0]['given_name'];
			$sBillingNameAddition = $aBillingInformation[0]['additional_name'];
			$sBillingLastName = $aBillingInformation[0]['family_name'];

			$aCombinedLastNameBilling = array($sBillingNameAddition, $sBillingLastName);
			$sCombinedLastNameBilling = implode(" ", $aCombinedLastNameBilling);

			$aOrderParams = array();

			// if no housenumber is filled or the housenumber is added behind the streetname we split hem so the framework get the correct data
			if(empty($sBillingHouseNumber))
			{
				$aBillingHouseInformation = idealcheckout_splitAddress($sBillingStreetName);

				$sBillingStreetName = $aBillingHouseInformation[0];
				if(!empty($aBillingHouseInformation[2]))
				{
					$sBillingHouseNumber = implode(" ", array($aBillingHouseInformation[1], $aBillingHouseInformation[2]));
				}
				else
				{
					$sBillingHouseNumber = $aBillingHouseInformation[1];
				}
			}

			$aOrderParams['payment_company'] = $sBillingCompany;
			$aOrderParams['payment_first_name'] = $sBillingFirstName;
			$aOrderParams['payment_last_name'] = $sCombinedLastNameBilling;
			$aOrderParams['payment_gender'] = '';
			$aOrderParams['payment_date_of_birth'] = '';
			$aOrderParams['payment_phone'] = '';
			$aOrderParams['payment_email'] = $sEmailAddress;
			$aOrderParams['payment_street_name'] = $sBillingStreetName;
			$aOrderParams['payment_street_number'] = $sBillingHouseNumber;
			$aOrderParams['payment_zipcode'] = $sBillingZip;
			$aOrderParams['payment_city'] = $sBillingCity;
			$aOrderParams['payment_country_code'] = $sBillingCountryCode;
			$aOrderParams['payment_country_name'] = '';

			// Get Shipping Data

			$oShipping = $oEntityManager->getStorage('commerce_shipment');	// grab the shipment storage
			$oShipments = $oShipping->loadMultipleByOrder($oOrder); // grab all the shipment for the current order (these are more than one)

			// Loop trough the shipping orders Grab from the first one the shipping profile and break out of the loop.
			foreach($oShipments as $oShipment)
			{
				$oShippingProfile = $oShipment->getShippingProfile();
				break;
			}

			$aShippingInformation = $oShippingProfile->address->getValue(); // Get the shipping address and shipping information
			
			// Set Shipping Data

			$sShippingCountryCode = $aShippingInformation[0]['country_code'];
			$sShippingCity = $aShippingInformation[0]['locality'];
			$sShippingZip = $aShippingInformation[0]['postal_code'];
			$sShippingStreetName = $aShippingInformation[0]['address_line1'];
			$sShippingHouseNumber = $aShippingInformation[0]['address_line2'];
			$sShippingCompany = $aShippingInformation[0]['organization'];
			$sShippingFirstName = $aShippingInformation[0]['given_name'];
			$sShippingNameAddition = $aShippingInformation[0]['additional_name'];
			$sShippingLastName = $aShippingInformation[0]['family_name'];

			$aCombinedLastNameShipping = array($sShippingNameAddition, $sShippingLastName);
			$sCombinedLastNameShipping = implode(" ", $aCombinedLastNameShipping);
			
			// if no housenumber is filled or the housenumber is added behind the streetname we split hem so the framework get the correct data
			if(empty($sShippingHouseNumber))
			{
				$aShippingHouseInformation = idealcheckout_splitAddress($sShippingStreetName);

				$sShippingStreetName = $aShippingHouseInformation[0];
				if(!empty($aShippingHouseInformation[2]))
				{
					$sShippingHouseNumber = implode(" ", array($aShippingHouseInformation[1], $aShippingHouseInformation[2]));
				}
				else
				{
					$sShippingHouseNumber = $aShippingHouseInformation[1];
				}
			}

			$aOrderParams['shipment_company'] = $sShippingCompany;
			$aOrderParams['shipment_first_name'] = $sShippingFirstName;
			$aOrderParams['shipment_last_name'] = $sCombinedLastNameShipping;
			$aOrderParams['shipment_gender'] = '';
			$aOrderParams['shipment_date_of_birth'] = '';
			$aOrderParams['shipment_phone'] = '';
			$aOrderParams['shipment_email'] = $sEmailAddress;
			$aOrderParams['shipment_street_name'] = $sShippingStreetName;
			$aOrderParams['shipment_street_number'] = $sShippingHouseNumber;
			$aOrderParams['shipment_zipcode'] = $sShippingZip;
			$aOrderParams['shipment_city'] = $sShippingCity;
			$aOrderParams['shipment_country_code'] = $sShippingCountryCode;
			$aOrderParams['shipment_country_name'] = '';

			// Set Product Data

			$aOrderParams['products'] = array();

			foreach($oOrder->getItems() as $key => $oOrderItem)
			{
				$aProduct = array();

				$fUnitPrice = $oOrderItem->getAdjustedUnitPrice()->getNumber();
				$aAdjustments = $oOrderItem->getAdjustments();

				$aProduct['code'] = $oOrderItem->getPurchasedEntityId();
				$aProduct['description'] = $oOrderItem->getTitle();
				$aProduct['quantity'] = round($oOrderItem->getQuantity());
				$aProduct['price_incl'] = round($fUnitPrice * 100);
				/*
					$aProduct['adjustments'] = $aAdjustments; // add the adjustments like promotion and VAT to the order params. This is not neccesary but for development it is added
				*/
				foreach($aAdjustments as $oAdjustment)
				{
					if(strcmp(strtolower($oAdjustment->getType()), 'tax') === 0)
					{
						$aProduct['price_exl'] = round(($fUnitPrice - $oAdjustment->getAmount()->getNumber()) * 100);
						$aProduct['vat'] = round($oAdjustment->getPercentage() * 100);
					}
					else
					{
						$aProduct['price_exl'] = round($fUnitPrice * 100);
						$aProduct['vat'] = 0;
					}
				}

				$aOrderParams['products'][] = $aProduct;
			}

			$oIdealcheckout->setOrderParams($aOrderParams);

			// Send Data to Database

			$oIdealcheckout->createTransactionQuery();
			$oIdealcheckout->executeQuery();

			// Redirect customer

			$sSetupUrl = $oIdealcheckout->getSetupUrl();

			return $this->buildRedirectForm
			(
				$form,
				$form_state,
				$sSetupUrl,
				array(),
				PaymentOffsiteForm::REDIRECT_POST
			);
			exit;
		}
	}
?>